/* Copyright (C) |Meso|Star> 2015-2017 (contact@meso-star.com)
 *
 * This software is a library whose purpose is to generate [pseudo] random
 * numbers and random variates.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty and the software's author, the holder of the
 * economic rights, and the successive licensors have only limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading, using, modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean that it is complicated to manipulate, and that also
 * therefore means that it is reserved for developers and experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and, more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms. */

#include "ssp_rng_c.h"

/*******************************************************************************
 * Exported state free random variates
 ******************************************************************************/
double
ssp_ran_exp(struct ssp_rng* rng, const double mu)
{
  ASSERT(rng);
  rng_cxx rng_cxx(*rng);
  RAN_NAMESPACE::exponential_distribution<double> distribution(mu);
  return distribution(rng_cxx);
}

double
ssp_ran_exp_pdf(const double x, const double mu)
{
  ASSERT(x >= 0);
  return mu * exp(-x * mu);
}

double
ssp_ran_gaussian
  (struct ssp_rng* rng,
   const double mu,
   const double sigma)
{
  ASSERT(rng);
  rng_cxx rng_cxx(*rng);
  RAN_NAMESPACE::normal_distribution<double> distribution(mu, sigma);
  return distribution(rng_cxx);
}

double
ssp_ran_gaussian_pdf(const double x, const double mu, const double sigma)
{
  const double tmp = (x - mu) / sigma;
  return 1.0 / (sigma * SQRT_2_PI) * exp(-0.5 * tmp * tmp);
}

double
ssp_ran_lognormal
  (struct ssp_rng* rng,
   const double zeta,
   const double sigma)
{
  ASSERT(rng);
  rng_cxx rng_cxx(*rng);
  RAN_NAMESPACE::lognormal_distribution<double> distribution(zeta, sigma);
  return distribution(rng_cxx);
}

double
ssp_ran_lognormal_pdf(const double x, const double zeta, const double sigma)
{
  const double tmp = log(x) - zeta;
  return 1.0 / (sigma * x * SQRT_2_PI) * exp(-(tmp*tmp) / (2*sigma*sigma));
}

